/**
 * Copyright (c) 2003-2007, Xith3D Project Group all rights reserved.
 * 
 * Portions based on the Java3D interface, Copyright by Sun Microsystems.
 * Many thanks to the developers of Java3D and Sun Microsystems for their
 * innovation and design.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 * 
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 * 
 * Neither the name of the 'Xith3D Project Group' nor the names of its
 * contributors may be used to endorse or promote products derived from this
 * software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) A
 * RISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE
 */
package org.jagatoo.loaders.models.obj;

import java.util.ArrayList;
import java.util.List;

/**
 * A single group within the model
 * 
 * @author Kevin Glass
 * @author Marvin Froehlich (aka Qudus)
 */
public class OBJGroup extends OBJFaceList
{
    private String name;
    private boolean isTopGroup;
    private List<OBJGroup> children = new ArrayList<OBJGroup>();
    
    public void setName( String name )
    {
        this.name = name;
    }
    
    public String getName()
    {
        return( name );
    }
    
    public boolean hasName()
    {
        return( ( name != null ) && ( name.length() > 0 ) );
    }
    
    public boolean isTopGroup()
    {
        return( isTopGroup );
    }
    
    public void addChild( OBJGroup group )
    {
        children.add( group );
    }
    
    public List<OBJGroup> getChildren()
    {
        return( children );
    }
    
    private OBJGroup( String name, boolean isTopGroup, List<float[]> verts, List<float[]> normals, List<float[]> texs )
    {
        super( verts, normals, texs );
        
        this.name = name;
        this.isTopGroup = isTopGroup;
    }
    
    public OBJGroup( String name, List<float[]> verts, List<float[]> normals, List<float[]> texs )
    {
        this( name, false, verts, normals, texs );
    }
    
    public static OBJGroup createTopGroup( List<float[]> verts, List<float[]> normals, List<float[]> texs )
    {
        return( new OBJGroup( "top", true, verts, normals, texs ) );
    }
}
