package uk.ac.soton.grophysics;

import java.lang.reflect.Method;
import java.util.ArrayList;
import javax.vecmath.Matrix4d;
import javax.vecmath.Matrix4f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import javax.vecmath.Vector3f;

import com.bulletphysics.collision.broadphase.AxisSweep3;
import com.bulletphysics.collision.dispatch.CollisionConfiguration;
import com.bulletphysics.collision.dispatch.CollisionDispatcher;
import com.bulletphysics.collision.dispatch.CollisionObject;
import com.bulletphysics.collision.dispatch.DefaultCollisionConfiguration;
import com.bulletphysics.collision.shapes.BoxShape;
import com.bulletphysics.collision.shapes.CollisionShape;
import com.bulletphysics.collision.shapes.CylinderShapeZ;
import com.bulletphysics.collision.shapes.SphereShape;
import com.bulletphysics.dynamics.DiscreteDynamicsWorld;
import com.bulletphysics.dynamics.DynamicsWorld;
import com.bulletphysics.dynamics.RigidBody;
import com.bulletphysics.dynamics.RigidBodyConstructionInfo;
import com.bulletphysics.dynamics.constraintsolver.Generic6DofConstraint;
import com.bulletphysics.dynamics.constraintsolver.RotationalLimitMotor;
import com.bulletphysics.dynamics.constraintsolver.SequentialImpulseConstraintSolver;
import com.bulletphysics.dynamics.constraintsolver.TranslationalLimitMotor;
import com.bulletphysics.linearmath.DefaultMotionState;
import com.bulletphysics.linearmath.MotionState;
import com.bulletphysics.linearmath.Transform;

import de.grogra.graph.GraphState;
import de.grogra.graph.impl.GraphManager;
import de.grogra.graph.impl.Node;
import de.grogra.imp3d.objects.Cylinder;
import de.grogra.imp3d.objects.Frustum;
import de.grogra.imp3d.objects.Null;
import de.grogra.imp3d.objects.Sphere;
import de.grogra.math.TMatrix4d;
import de.grogra.rgg.Library;
import de.grogra.turtle.F;


public class PhysicsModel extends Node {
	/**
	 * auto generated by eclipse
	 */
	private static final long serialVersionUID = -7481390093150351783L;
	// controllers for the world dynamics
	protected DynamicsWorld dynamicWorld;
	private CollisionConfiguration collisionConfiguration;
	private CollisionDispatcher dispatcher;
	// The body representing the ground
	RigidBody groundBody ;
	float groundDepth = 10.f;
	Transform groundJointFrame;

//	public boolean debugRendering;
//	//enh:field getter setter
   
	public boolean debugText;
	//enh:field getter setter

	// TODO look at using a weak hash map to maintain the list of objects
	// TODO add to enhanced fields when persistence works
	private ArrayList<MassObject> massObjects;
	//enh:field attr=Attributes.MASSOBJECTS getter setter

	final PhysicsVisitor visitor = new PhysicsVisitor();
	public final PhysicsUtils utils = new PhysicsUtils();
	private int autoIndex = 0;

	public PhysicsModel() {
		super();
		init();
		massObjects = new ArrayList<MassObject>();
//		debugRendering = false;
		debugText = false;
		initVisitor();
	}
	
	protected void init() {
		// create new collision control for the world
		collisionConfiguration = new DefaultCollisionConfiguration();
		dispatcher = new CollisionDispatcher(collisionConfiguration);
		Vector3f worldAabbMin = new Vector3f(-1000,-1000,-1000);
		Vector3f worldAabbMax = new Vector3f(1000,1000,1000);
		AxisSweep3 overlappingPairCache = new AxisSweep3(worldAabbMin, worldAabbMax);
		SequentialImpulseConstraintSolver solver = new SequentialImpulseConstraintSolver();
		//create the world
		dynamicWorld = new DiscreteDynamicsWorld(dispatcher, overlappingPairCache, solver, collisionConfiguration);
		// add gravity force ( applies to all objects )
		dynamicWorld.setGravity(new Vector3f(0,0,-9.81f));
		dynamicWorld.getDispatchInfo().allowedCcdPenetration = 0f;
		// add the floor
		addGround(100.f, groundDepth/2);
		// create a list of objects to manage
	}
	
	public void initVisitor() {
		GraphState gs = Library.graph().getMainState();
		Matrix4d m = new Matrix4d();
		m.setIdentity();
		visitor.init(gs, m, this);
	}

	protected void addGround(float size, float depth) {
		//create the ground shape
		CollisionShape groundShape = new BoxShape(new Vector3f(size, size, depth));
//		CollisionShape groundShape = new StaticPlaneShape(new Vector3f(0, 1, 0), 50);
		groundShape.setUserPointer("Ground");
		// set its location
		Transform groundTransform = new Transform();
		groundTransform.setIdentity();
		// place the ground box so that the surface is height 0 
		groundTransform.origin.set(new Vector3f(0.f, 0.f, -depth));
		// zero mass = fixed location
		float mass = 0f;
		Vector3f localInertia = new Vector3f(0, 0, 0);
	    DefaultMotionState myMotionState = new DefaultMotionState(groundTransform);
		RigidBodyConstructionInfo rbInfo = new RigidBodyConstructionInfo(
					mass, myMotionState, groundShape, localInertia);
		groundBody = new RigidBody(rbInfo);
		// restitution higher = more bouncy
	    //groundBody.setRestitution(0.3f);
	    groundBody.setRestitution(0.7f);
//	    groundBody.setFriction(0.1f);
	    groundBody.setDamping(0.5f, 0.5f);
//	    groundBody.setDeactivationTime(0.01f);
		dynamicWorld.addRigidBody(groundBody);
		// transform for a joint on the ground
		groundJointFrame = new Transform();
		groundJointFrame.setIdentity();
		Vector3f groundOrigin = new Vector3f(0, 0, groundDepth/2);
		groundJointFrame.origin.set(groundOrigin);	
	}
	
	
	// add an object with automatic name 
	public void add(Object o) {
		String name = "AutoItem"+autoIndex++;
		float mass = getMass(o);
		add(name, o, mass);
	}

	
	// add an object ( just adds to a list ) 
	public void add(String name, Object o) {
		float mass = getMass(o);
		add(name, o, mass);
	}
	
	// add an object with automatic name 
	public void add(Object o, float mass) {
		String name = "AutoItem"+autoIndex++;
		add(name, o, mass);
	}

	// add an object ( just adds to a list ) 
	public void add(String name, Object o, float mass) {
		if (o == null) {
			Library.println("WARNING: Can't add a null object to the PhysicsModel");
		} else {
			MassObject m = new MassObject(name, o, mass);
			if (debugText) Library.println("PhysicsModel: Adding "+name+" mass "+mass);
			massObjects.add(m);
		}
	}

	public void remove(Object o) {
		MassObject mo = getMassObject(o);
		if ( mo != null) {
			massObjects.remove(mo);
		}
	}

	public void setMass(Object o, float mass) {
		MassObject mo = getMassObject(o);
		if ( mo != null) {
			mo.mass = mass;
		}
	}

	
	public void updateReference(String name, Object o) {
		// find the mass object with the matching name
		for ( int i = 0; i < massObjects.size(); i++) {
			MassObject m = massObjects.get(i);
			if (name.equals(m.getName())) {
				m.setObject(o);
				Library.println("Updating "+name);
				break;
			}
		}
	}
	
   private void updateMass(MassObject m) {
	   if (debugText) Library.println("\nStart updateMass");
	   Object o = m.getObject();
	   float mass =0;
	   if (o instanceof FlexNode) {
		   FlexNode f = (FlexNode) o;
		   mass = f.getMass();
	   } else {
		   mass = getMass(o);
	   }
	   m.setMass(mass);
   }
	
	/*
	 * get the mass of an object if it has a getMass method
	 * returns 0 if not.
	 */
	public float getMass(Object o) {
		float mass = 0;
		Class<?> c = o.getClass();
		try {
			Method method = c.getMethod("getMass", (Class<?>) null);
			Object ret = method.invoke(null);
			if (method.getReturnType().equals(float.class)) {
				mass = (Float) ret;
			}
		} catch (Exception e) {
			Library.println("WARNING: can't find mass for "+o.getClass().getName());
			Library.println(e.getMessage());
		}
		return mass;
	}
	

	
	/*
	 *  create a physical body for an object 
	 */
	protected void addDeferred(MassObject m) {
		Object o = m.getObject();
		RigidBody body = null;
		CollisionShape shape = null;
		if (o == null) {
			Library.println("WARNING: Adding a null object to the PhysicsModel");
		} else if (o instanceof de.grogra.imp3d.objects.Sphere) {
			if (debugText) Library.println("Adding a Sphere to the PhysicsModel");
			de.grogra.imp3d.objects.Sphere sphere = (de.grogra.imp3d.objects.Sphere) o;
			//create a JBullet sphere shape
			shape = new SphereShape(sphere.getRadius());
			// reference the object in case we need it
			shape.setUserPointer(o);
			// create a body
			body = createAndAddBody(m, shape);
//			join(sphere, m, body);

		} else if (o instanceof FlexNode) {
			if (debugText) Library.println("Adding a FlexNode to the PhysicsModel, mass "+m.getMass());
			FlexNode flexNode = (FlexNode)(o);
			// length less a margin at the joint
	//		float bodyLength = flexNode.getLength() /2f - flexNode.getRadius();
			// true body length
			float r = flexNode.getRadius();
			float l = flexNode.getLength();
			body = createCylinder(m, o, l, r);
			// try to join to something
			join(flexNode, m, body);

		} else if (o instanceof Frustum) {
			if (debugText) Library.println("Adding a Frustum to the PhysicsModel (as Cylinder), mass "+m.getMass());
			Frustum frust = (Frustum)(o);
			float r = frust.getTopRadius();
			float rb = frust.getBaseRadius();
			// use larger radius for cylinder
			if  (rb > r) r = rb;
			float l = frust.getLength();
			body = createCylinder(m, o, l, r);
//			join(frust, m, body);

		} else if (o instanceof Cylinder) {
			if (debugText) Library.println("Adding a Cylinder to the PhysicsModel, mass "+m.getMass());
			Cylinder cyl = (Cylinder)(o);
			float r = cyl.getRadius();
			float l = cyl.getLength();
			body = createCylinder(m, o, l, r);
//			join(cyl, m, body);
			
		} else if (o instanceof F) {
			if (debugText) Library.println("Adding a Cylinder to the PhysicsModel, mass "+m.getMass());
			F fcyl = (F)(o);
			float r = fcyl.diameter/2;
			float l = fcyl.length;
			body = createCylinder(m, o, l, r);
			// try to join to something
//			join(fcyl, m, body);
			
		} else {
			Library.println("Can't add a "+o.getClass()+" to the PhysicsModel (yet)");
			Library.println("	Possible Classes:");
			Library.println("      "+o.getClass().getSuperclass()+" (super)");
			Class<?> [] carr = o.getClass().getClasses();
			for (int i = 0; i < carr.length; i++) {
				Library.println("      "+carr[i].getName());
			}
		}
	}
	
    protected void updatePosition(MassObject m) {
    	if (debugText) Library.println("\nStart updatePosition");
//		Library.println("Local Before enter "+ getCurrentTransformation());

		// get the physics object
		CollisionObject o = m.getBody();
		// get the groimp object
		Object nodeObject = m.getObject() ;
		if (nodeObject == null) {
			Library.println("WARNING: MassObject reference object is NULL");
		} else if (nodeObject instanceof Node) {
			Node node = (Node) nodeObject;
			FlexNode flexNode = null;
			FlexNode parentFlexNode = null;
			if (node instanceof FlexNode) {
				flexNode = (FlexNode) node;
				parentFlexNode = flexNode.getParentFlexNode();
			}		
			
			// get the objects position in the physics world
			Transform worldTransform = new Transform();
			worldTransform.setIdentity();
			o.getWorldTransform(worldTransform);
			// get the current position matrix
			Matrix4d pm = utils.getMatrix4d(worldTransform);
			if (debugText) Library.println("Physics world transform ="+pm);
			// get the offset as a vector
			Point3d offsetP = m.getOffsetCoG();
			Vector3d noffset = new Vector3d();
			noffset.negate(offsetP);
			// get the required position with offset
			Matrix4d nm = utils.offsetMatrix(pm, noffset);
			if (debugText) Library.println("compensated for centre of mass ="+nm);
			// nm is now the desired position for GroIMP
			
			// store it for use by its children
			m.setCurrentGlobalTransform(nm);
			if (debugText) Library.println("set current global "+m.getName()+" to "+nm);
			

			// look for a predecessor node
//			Node prevNode = node.getPredecessor();
			Matrix4d im;
			if (flexNode == null || parentFlexNode == null) {
				// get the original position in GroImp
				im = new Matrix4d(m.getInitialTransform());
//				im = m.getParentFrameTransform();
				if (im == null) {
					Library.println("ERROR: Parent transform for "+m.getName()+" is NULL");
				}
				if (debugText) Library.println("Initial transform with no parent ="+im);
			} else {
				if (debugText) Library.println("Parent FlexNode "+parentFlexNode.getName());
				
				// get the massObject of the parent
				MassObject parentMassObject = getMassObject(parentFlexNode);
				if (parentMassObject == null) {
					Library.println("WARNING: Can't find a MassObject for the parent");
					im = new Matrix4d(m.getInitialTransform());
				} else {
					// get the current position of the parent (updated)
					im = parentMassObject.getCurrentGlobalTransform();
					if (debugText) Library.println("Parent ("+parentMassObject.getName() +") current transform ="+im);
					

					// retrieve the transform from parent pre to post 
					Matrix4d dm = m.getParentFrameTransform();
					if (debugText) Library.println("Joint offset -\n"+dm.toString());
					
					// add it to the parent position to get the new origin 
					//  of the current object with no transformation
					im.mul(dm);
					if (debugText) Library.println("Parent with joint offset -\n"+im.toString());

				}
			}
			
			// calculate the difference from the start position
			Matrix4d delta = utils.getDifference(nm, im);
			if (debugText) Library.println("Change matrix ="+delta);
			// set the transform in groIMP to match JBullet object
			TMatrix4d deltaTM = new TMatrix4d(delta);
			((Null) nodeObject).setTransform(deltaTM);
			
//			Library.println("Local After enter "+ getCurrentTransformation());
//			Matrix4d localAfter = ((Null) nodeObject).getLocalTransformation();
//			Library.println("LocalBefore ="+localBefore);
//			Library.println("Local After ="+localAfter);
		}
		if (debugText) Library.println("End updatePosition");
    }


	
	protected RigidBody createCylinder(MassObject mo, Object o, float l, float r) {
		RigidBody body = null;
		CollisionShape shape = null;
		// use radius of cylinder and offset for CoG
		Vector3f extents = new Vector3f(r, r, l/2);
		//create a JBullet sphere shape
		shape = new CylinderShapeZ(extents);
		// reference the object in case we need it
		shape.setUserPointer(o);
		// set the CoG Offset for a cylinder
		Point3d offCoG = new Point3d(0, 0, l/2);
		mo.setOffsetCoG(offCoG);
		// create a body
		body = createAndAddBody(mo, shape);
		return body;
	}

	// join a flex node to something
	protected void join(FlexNode flexNode, MassObject massObject, RigidBody body) {
		
		// find parent
		MassObject parentMassObject = getParentMassObject(flexNode);
		if (parentMassObject == null) { 

			//no parent object so join to ground
			if (debugText) Library.println("Join to ground");

			// the joint frame on the ground should be the point at the
			// world coordinates of the bottom of the object

			// get position relative to parent object ( origin in this case )
			Matrix4d parentTransform = massObject.getParentFrameTransform();
			if (parentTransform == null) {
				parentTransform = Library.transformation(flexNode).toMatrix4d();
				massObject.setParentFrameTransform(parentTransform);
				if (debugText) Library.println("Set parent transform to "+parentTransform);
			} else {
				if (debugText) Library.println("  parent transform is "+parentTransform);
			}


			// get the joint frame at the top of the ground
			Matrix4f gm4f = new Matrix4f();
			// NB groundJointFrame is the top of the ground object at the origin
			groundJointFrame.getMatrix(gm4f);
			Matrix4d groundJoint = new Matrix4d(gm4f);
			if (debugText) Library.println("Ground Joint Frame -\n"+groundJoint.toString());
			groundJoint.mul(parentTransform);
			if (debugText) Library.println("Anchor Frame -\n"+groundJoint.toString());
			Transform groundAnchorFrame = new Transform();
			groundAnchorFrame.set(new Matrix4f(groundJoint));
			
			joinToBodyFrame(flexNode, groundAnchorFrame, body, groundBody);
				
		} else {
//
			// Has a parent mass object
			Object parentObj = parentMassObject.getObject();
			// see of the object is a flexNode
			if (! (parentObj instanceof FlexNode)) {
				if (! (parentObj instanceof Node)) {
					// Parent is not a Node
					Library.println("WARNING: Can't find a Node for the parent");
					// set parent Transform to the global position
					Matrix4d parentTransform = Library.transformation(flexNode).toMatrix4d();
					massObject.setParentFrameTransform(parentTransform);
				} else {
					// Parent is a Node
					Node parentNode = (Node) parentObj;
					// get the body of the parent and its transform
					RigidBody parentBody = (RigidBody) parentMassObject.getBody();
					if (debugText) Library.println("Store transform from "+parentMassObject.getName() +" to "+massObject.getName());
					
					// get position relative to parent object
					Matrix4d parentTransform = massObject.getParentFrameTransform();
					if (parentTransform == null) {
						parentTransform = calcParentTransform(flexNode, parentNode);
						massObject.setParentFrameTransform(parentTransform);
						if (debugText) Library.println("Set parent transform to "+parentTransform);
					} else {
						if (debugText) Library.println("  parent transform is "+parentTransform);
					}
					// position of the parent
					Matrix4d parentGlobal = Library.transformation(parentNode).toMatrix4d();
					// get the CoG offset of the parent
					Point3d offsetP = parentMassObject.getOffsetCoG();
					Matrix4d parentCoG = utils.offsetMatrix(parentGlobal, offsetP);
					if (debugText) Library.println("parent Centre of Mass "+parentCoG.toString());
										
					// get the frame of the object in GroIMP
					Matrix4d tm4 = Library.transformation(flexNode).toMatrix4d();
					if (debugText) Library.println("Object Transformation "+tm4.toString());
					// get a transform for the object frame relative to the parent centre
					Matrix4d jointFrame = utils.getDifference(tm4, parentCoG);
					if (debugText) Library.println(" jointFrame "+jointFrame.toString());

					
//					Vector3d noffset = new Vector3d();
//					noffset.negate(offsetP);
//					// get the required position with offset
//					Matrix4d jointFrame = utils.offsetMatrix(parentTransform, noffset);
////					if (debug) Library.println("compensated for centre of mass ="+jointFrame);
//
					if (debugText) Library.println("Anchor Frame -\n"+jointFrame.toString());
					Transform anchorFrame = new Transform();
					anchorFrame.set(new Matrix4f(jointFrame));

					if (debugText) Library.println("Join to parent "+parentMassObject.getName());
					// joint to a fixed point relative to the object's centre
					joinToBodyFrame(flexNode, anchorFrame,  body, parentBody);
				}
			} else {
				// parent is another FlexNode
				FlexNode parentFlexNode = (FlexNode) parentObj;
				if (debugText) Library.println("Store transform from "+parentMassObject.getName() +" to "+massObject.getName());

				// get position relative to parent object
				Matrix4d parentTransform = massObject.getParentFrameTransform();
				if (parentTransform == null) {
					parentTransform = calcParentTransform(flexNode, parentFlexNode);
					massObject.setParentFrameTransform(parentTransform);
					if (debugText) Library.println("Set parent transform to "+parentTransform);
				} else {
					if (debugText) Library.println("  parent transform is "+parentTransform);
					//TODO this should really be recalculated because other groimp 
					//nodes between this and the parent may have changed 
				}
				
				if (debugText) Library.println("Join to parent "+parentMassObject.getName());
				// get the body of the parent and its transform
				RigidBody parentBody = (RigidBody) parentMassObject.getBody();
				// make joints
				joinToParent(flexNode, parentFlexNode, body, parentBody);

			}
		}

	}

    /**
     * calculate a transform representing the position relative to the 
     * end of the parent stem
     * using GroIMP positions
     * 
     * @param node
     * @param parent
     * @return
     */
    Matrix4d calcParentOffsetFrame(FlexNode node, FlexNode parent) {
		// calculate the top joint offset (again)
		// get the frame of the parent in GroIMP
		Matrix4d pm4 = Library.transformation(parent).toMatrix4d();
		if (debugText) Library.println("Parent Transformation -\n"+pm4.toString());
		Vector3d offset = parent.getLengthVector();
		Matrix4d jointFrame = utils.offsetMatrix(pm4, offset);
		if (debugText) Library.println("Parent End -\n"+jointFrame.toString());

		// get the frame of the object in GroIMP
		Matrix4d tm4 = Library.transformation(node).toMatrix4d();
		if (debugText) Library.println("Object Transformation -\n"+tm4.toString());
		// get a transform for the object frame relative to the parent anchor frame
		Matrix4d dm = utils.getDifference(tm4, jointFrame);
		if (debugText) Library.println(" calcParentOffsetFrame -\n"+dm.toString());
		return dm;
    }

	
	
	/*
	 * join a FlexNode to a position in another pody given by an anchor frame
	 */
	protected void joinToBodyFrame(FlexNode flexNode, Transform jointFrame, RigidBody body, RigidBody parentBody) {

		// transform for joint on bottom
		Transform bottomJointFrame = flexNode.getBottomJointFrame();
		// use tm4f to print transforms
		Matrix4f tm4f = new Matrix4f(); 
		bottomJointFrame.getMatrix(tm4f);
		if (debugText) Library.println("Bottom Joint Frame -\n"+tm4f.toString());

		// create a new joint
		Generic6DofConstraint joint = new Generic6DofConstraint(parentBody, body, jointFrame, bottomJointFrame, false);
		flexNode.setJoint(joint);
		dynamicWorld.addConstraint(joint, true);
		// set up joint stiffness
		initMotors(joint, flexNode);
	}

	
	protected void joinToParent(FlexNode flexNode, FlexNode parentFlexNode, RigidBody body, RigidBody parentBody) {
		if (debugText) Library.println("Join to parent "+parentFlexNode.getName());
//		// get joint anchor of previous flexNode
		Transform topJointFrame = parentFlexNode.getTopJointFrame();
		// the offset from the top of the parent to its top joint frame
		Matrix4d parentOffset = calcParentOffsetFrame(flexNode, parentFlexNode);
		// get the joint frame relative to the parent CoG
		Transform parentTransform = new Transform(new Matrix4f(parentOffset));
		topJointFrame.mul(parentTransform);

		// NB the top joint frame an also be found from the parent current position (CoG)
		// and the flexNode parentTransform
		
		// get joint anchor on this node
		Transform bottomJointFrame = flexNode.getBottomJointFrame();
		
		if (debugText) { 
			Matrix4f jmf = new Matrix4f();
			topJointFrame.getMatrix(jmf);
			Library.println("PhysicsModel.joinToParent(): Top Joint Frame  -\n"+jmf);
			bottomJointFrame.getMatrix(jmf);
			Library.println("PhysicsModel.joinToParent(): Bottom Joint Frame  -\n"+jmf);
		}

		// join to parent
		Generic6DofConstraint joint = new Generic6DofConstraint(parentBody, body, topJointFrame, bottomJointFrame, false);
		flexNode.setJoint(joint);
		
		dynamicWorld.addConstraint(joint, true);
		// set up joint stiffness
		initMotors(joint, flexNode);

	}
	
	
	
	/**
	 * set up the joint motors from the stiffness parameters in the flexNode
	 * 
	 * @param joint
	 * @param flexNode
	 */
	private void initMotors(Generic6DofConstraint joint, FlexNode flexNode) {
		if (debugText) Library.println("initMotors() ");
		// set stiffness with motors
		RotationalLimitMotor motor;
		for ( int i = 0; i < 3; i++) {
			motor = joint.getRotationalLimitMotor(i);
	    	motor.loLimit = -1e-5f; // need to set limits or joint falls freely
	    	motor.hiLimit = 1e-5f;	
	    	//motor.maxMotorForce = 0.5f;
	    	motor.ERP = 0.7f; // default is 0.5
	    	motor.damping = flexNode.getRotDamping();
	    	motor.bounce = flexNode.getRotBounce();
	    	motor.limitSoftness = flexNode.getStiffness(i);
			if (debugText) Library.println(" stiffness("+i+")="+flexNode.getStiffness(i));
	    	motor.enableMotor = true;	
		}
		if (debugText) Library.println(" Rotational Damping="+flexNode.getRotDamping());
		if (debugText) Library.println(" Rotational bounce ="+flexNode.getRotBounce());
		if (debugText) Library.println(" Linear Damping    ="+flexNode.getLinDamping());
		if (debugText) Library.println(" Linear Bounce     ="+flexNode.getLinBounce());

		TranslationalLimitMotor tMotor = joint.getTranslationalLimitMotor();
		tMotor.limitSoftness = flexNode.getStiffness(3);
    	tMotor.damping = flexNode.getLinDamping();
    	// imits default is zero
 //   	tMotor.lowerLimit.set(new Vector3f(-1e-6f, -1e-6f, 0));
 //   	tMotor.upperLimit.set(new Vector3f(1e-6f, 1e-6f, 0.001f));
    	tMotor.restitution = flexNode.getLinBounce();;	
 		if (debugText) Library.println(" stiffness(3)="+flexNode.getStiffness(3));
	}
	
	//TODO get material properties too
	/*
	 * create a body for the MassObject with the supplied shape
	 * and add it to the physical world
	 */
	private RigidBody createAndAddBody(MassObject m, CollisionShape shape) {
		// create a body using the mass of the MassObject
		RigidBody body = createRigidBody(shape, m.getMass());
		m.setShape(shape);
		m.setBody(body);
		// initialise the position
		initPosition(m);
		// set physical parameters
//		body.setRestitution(0.3f);
		body.setRestitution(0.7f);
//	    body.setFriction(0.5f);
	    body.setDamping(0.3f, 0.9f);
	   
	    // TODO move to visitor?
		// add it to the world
		dynamicWorld.addRigidBody(body);
		// reference the original object in the physics object
		m.setInitialised(true);
		return body;
	}

	/**
	 * Create a rigid body which can be added to the JBullet world
	 * @param shape
	 */
	private RigidBody createRigidBody(CollisionShape shape, float mass) {
		Vector3f localInertia = new Vector3f(0, 0, 0);
		if (mass != 0f) {
			shape.calculateLocalInertia(mass, localInertia);
		}
		// shape.setMargin(0.1f);
		// set a default position
		Transform startTransform = new Transform();
		startTransform.setIdentity();
		MotionState ms = new DefaultMotionState(startTransform);
		// create a body from the shape 
		RigidBodyConstructionInfo rbInfo = new RigidBodyConstructionInfo(mass, ms, shape, localInertia);
		RigidBody body = new RigidBody(rbInfo);
		return body;
	}

    private void initPosition(MassObject m) {
//		if (debug) Library.println("\nStart initPosition");
    	CollisionObject co = m.getBody();
		Object nodeObject = m.getObject() ;
		if (nodeObject == null) {
			Library.println("WARNING: MassObject reference object is NULL");
		} else if (nodeObject instanceof Node) {
			Node n = (Node) nodeObject;
			String typeName = getNodeName(n);
			// get the position as a full transform matrix
			Matrix4d imd = Library.transformation(n).toMatrix4d();
			if (debugText) Library.println("Placing "+typeName +" at transform "+imd);
			// save the current position of the object
			m.setCurrentGlobalTransform(imd);
			// save the original position of the object first time
			Matrix4d initialTransform = m.getInitialTransform();
			if (initialTransform == null) {
				initialTransform = new  Matrix4d(imd);
				m.setInitialTransform(initialTransform);
			}

			// centre of mass
			Point3d offset = m.getOffsetCoG();
			if (debugText) Library.println("Centre of mass offset "+offset);

			// Include the offset for the centre of mass
			Matrix4d tmd = utils.offsetMatrix(imd, offset);
			if (debugText) Library.println("Physics world transform ="+tmd);
			
			// set the initial transform
			Transform worldTransform = new Transform();
			worldTransform.set(new Matrix4f(tmd));
//			Library.println("Set origin  "+worldTransform.origin);
//			Library.println("Set basis   "+worldTransform.basis);
			co.setWorldTransform(worldTransform);
			// save the original position of the object in the physics world
//			m.setInitialPhysicsTransform(worldTransform);
		}
//		Library.println("End initPosition");
    }
    
    /**
     * calculate a transform representing the position relative to the parent
     * using GroIMP positions
     * 
     * @param node
     * @param parent
     * @return
     */
    Matrix4d calcParentTransform(Node node, Node parent) {
		// calculate the top joint offset (again)
		// get the frame of the parent in GroIMP
		Matrix4d pm4 = Library.transformation(parent).toMatrix4d();
		if (debugText) Library.println("Parent Transformation -\n"+pm4.toString());
		// get the frame of the object in GroIMP
		Matrix4d tm4 = Library.transformation(node).toMatrix4d();
		if (debugText) Library.println("Object Transformation -\n"+tm4.toString());
		// get a transform for the object frame relative to the parent anchor frame
		Matrix4d dm = utils.getDifference(tm4, pm4);
		if (debugText) Library.println(" calcParentTransform -\n"+dm.toString());
		return dm;
    }

    /*
     * Uses the latest position of the parent 
     * which is updated during the visitor tree walk
     */
    Matrix4d calcUpdatedParentTransform(Node node, MassObject parent) {
		// calculate the top joint offset (again)
		// get the frame of the parent in GroIMP
		Matrix4d pm4 = parent.getCurrentGlobalTransform();
		if (debugText) Library.println("Parent Transformation -\n"+pm4.toString());
		// get the frame of the object in GroIMP
		Matrix4d tm4 = Library.transformation(node).toMatrix4d();
		if (debugText) Library.println("Object Transformation -\n"+tm4.toString());
		// get a transform for the object frame relative to the parent anchor frame
		Matrix4d dm = utils.getDifference(tm4, pm4);
		if (debugText) Library.println(" calcParentTransform -\n"+dm.toString());
		return dm;
    }

    
    /*
     * Give a meaningful name for an object 
     * defaults t the class name
     */
	public String getNodeName(Node n) {
		String result = "Null";
		if (n != null) {
			result = n.getClass().getSimpleName();
			if (n instanceof Sphere) {
				result = "Sphere";
			} else if (n instanceof FlexNode) {
				result = "FlexNode";
			} else if (n instanceof Cylinder) {
				result = "Cylinder";
			}
			if ( n.getName() != null) {
				result += " "+n.getName();
			}
		}
		return result;
	}

	/**
	 * perform a default 0.1s time step
	 */
	public void step() {
		step(0.1f);
	}	
	
	
	/**
	 * perform a time step
	 */
	public void step(float stepTime) {
		visitor.setDebug(debugText);
//    	Library.println("Move");
		// update positions using visitor
		GraphManager graph = Library.graph();
		graph.accept(null, visitor, null );

		//Library.println("Total objects "+dynamicWorld.getNumCollisionObjects());
		// each frame, do a time step on the Scene
//    	if (dynamicWorld.getNumCollisionObjects() > 1) {
    		dynamicWorld.stepSimulation(stepTime);
//    	}
	}
	

	// TODO this could be worked out during the time step
	/**
	 * true if any objects are still moving
	 */
	public boolean isActive() {
		boolean result = false;
		MassObject mo;
		CollisionObject body;
		for (int i = 0; i < massObjects.size(); i++) { 
			mo = massObjects.get(i);
			body = mo.getBody();
			if (body == null || mo.getBody().isActive()) {
				result = true;
				break;
			}
		}
		return result;
	}
	
	// TODO this could be worked out during the time step
	/**
	 * returns the number of objects are still moving
	 */
	public int activeCount() {
		int result = 0;
		MassObject mo;
		CollisionObject body;
		for (int i = 0; i < massObjects.size(); i++) { 
			mo = massObjects.get(i);
			body = mo.getBody();
			if (body == null || mo.getBody().isActive()) {
				result++;
			}
		}
		return result;
	}
	
	public void clear() {
		if (debugText) Library.println("\nPhysicsModel: clearing all objects");
		MassObject mo;
		for (int i = 0; i < massObjects.size(); i++) { 
			mo = massObjects.get(i);
			mo.setInitialised(false);
		}
		dynamicWorld.destroy();
		init();
////		dynamicWorld.clearForces();
////		// remove constraints
//		for ( int i = 0; i < dynamicWorld.getNumConstraints(); i++) {
//			TypedConstraint constraint = dynamicWorld.getConstraint(i);
//			dynamicWorld.removeConstraint(constraint);
//		}
////		// remove objects
//		MassObject mo;
//		CollisionObject body;
//		for (int i = 0; i < massObjects.size(); i++) { 
//			mo = massObjects.get(i);
//			body = mo.getBody();
//			if (body != null && body != groundBody) {
//				dynamicWorld.removeCollisionObject(body);
//			}
//			mo.setInitialised(false);
//		}
	}
	
	/**
	 * Find the previous FlexNode in the current branch
	 * returns null if nothing is found
	 * @return FlexNode
	 */
	public FlexNode getPreviousFlexNode(Node node) {
		FlexNode previous = null;
		Node pred = node.getPredecessor();
		while (pred != null) {
			if (pred instanceof FlexNode) {
				previous = (FlexNode) pred;
				break;
			}
			pred = pred.getPredecessor();
		}
		return previous;
	}

	/**
	 * Find the previous FlexNode in the tree by searching the branch first, 
	 * then hierarchically through it's parents
	 * returns null if nothing is found
	 * @return FlexNode
	 */
	public FlexNode getParentFlexNode(Node node) {
		FlexNode parentFlexNode = getPreviousFlexNode(node);
		if  (parentFlexNode == null) {
			// no more FlexNodes in this branch so look for a parent node
			Node parent = node.getAxisParent();
find:		while (parent != null) {
				if (parent instanceof FlexNode) {
					parentFlexNode = (FlexNode) parent;
					break find;
				} else {
					// try the predecessors of the parent
					Node pred = parent.getPredecessor();
					while (pred != null) {
						if (pred instanceof FlexNode) {
							parentFlexNode = (FlexNode) pred;
							break find;
						}
						pred = pred.getPredecessor();
					}
				}
				// try the previous parent
				parent = parent.getAxisParent();
			}
		}
		return parentFlexNode;
	}

	// Find a MassObject from a node
	MassObject getMassObject(Node node) {
		MassObject result = null;
		for ( int i = 0; i < massObjects.size(); i++) {
			MassObject m = massObjects.get(i);
			Object o = m.getObject();
			if (o instanceof Node) {
				Node n = (Node) o;
				if (n.equals(node)) {
					result = m;
					break;
				}
			}
		}
		return result;
	}
	// Find a MassObject relating to an object
	MassObject getMassObject(Object obj) {
		MassObject result = null;
		for ( int i = 0; i < massObjects.size(); i++) {
			MassObject m = massObjects.get(i);
			Object o = m.getObject();
			if (o .equals(obj)) {
				result = m;
				break;
			}
		}
		return result;
	}

	
	/**
	 * Find the previous FlexNode in the current branch
	 * returns null if nothing is found
	 * @return MassObject
	 */
	public MassObject getPreviousMassObject(Node node) {
		MassObject previous = null;
		Node pred = node.getPredecessor();
		while (pred != null) {
			previous = getMassObject(pred);
			if (previous != null) {
				break;
			}
			pred = pred.getPredecessor();
		}
		return previous;
	}

	/**
	 * Find the previous FlexNode in the tree by searching the branch first, 
	 * then hierarchically through it's parents
	 * returns null if nothing is found
	 * @return MassObject
	 */
	public MassObject getParentMassObject(Node node) {
		MassObject parentMassObject = getPreviousMassObject(node);
		if  (parentMassObject == null) {
			// no more FlexNodes in this branch so look for a parent node
			Node parent = node.getAxisParent();
find:		while (parent != null) {
				parentMassObject= getMassObject(parent);
				if (parentMassObject != null) {
					break find;
				} else {
					// try the predecessors of the parent
					Node pred = parent.getPredecessor();
					while (pred != null) {
						parentMassObject = getMassObject(pred);
						if (parentMassObject != null) {
							break find;
						}
						pred = pred.getPredecessor();
					}
				}
				// try the previous parent
				parent = parent.getAxisParent();
			}
		}
		return parentMassObject;
	}

	
	
    //enh:insert
//enh:begin
// NOTE: The following lines up to enh:end were generated automatically

	public static final NType $TYPE;

	public static final NType.Field debugText$FIELD;
	public static final NType.Field massObjects$FIELD;

	private static final class _Field extends NType.Field
	{
		private final int id;

		_Field (String name, int modifiers, de.grogra.reflect.Type type, de.grogra.reflect.Type componentType, int id)
		{
			super (PhysicsModel.$TYPE, name, modifiers, type, componentType);
			this.id = id;
		}

		@Override
		public void setBoolean (Object o, boolean value)
		{
			switch (id)
			{
				case 0:
					((PhysicsModel) o).debugText = (boolean) value;
					return;
			}
			super.setBoolean (o, value);
		}

		@Override
		public boolean getBoolean (Object o)
		{
			switch (id)
			{
				case 0:
					return ((PhysicsModel) o).isDebugText ();
			}
			return super.getBoolean (o);
		}

		@Override
		protected void setObjectImpl (Object o, Object value)
		{
			switch (id)
			{
				case 1:
					((PhysicsModel) o).massObjects = (ArrayList<MassObject>) value;
					return;
			}
			super.setObjectImpl (o, value);
		}

		@Override
		public Object getObject (Object o)
		{
			switch (id)
			{
				case 1:
					return ((PhysicsModel) o).getMassObjects ();
			}
			return super.getObject (o);
		}
	}

	static
	{
		$TYPE = new NType (new PhysicsModel ());
		$TYPE.addManagedField (debugText$FIELD = new _Field ("debugText", _Field.PUBLIC  | _Field.SCO, de.grogra.reflect.Type.BOOLEAN, null, 0));
		$TYPE.addManagedField (massObjects$FIELD = new _Field ("massObjects", _Field.PRIVATE  | _Field.SCO, de.grogra.reflect.ClassAdapter.wrap (ArrayList.class), null, 1));
		$TYPE.declareFieldAttribute (massObjects$FIELD, Attributes.MASSOBJECTS);
		$TYPE.validate ();
	}

	@Override
	protected NType getNTypeImpl ()
	{
		return $TYPE;
	}

	@Override
	protected de.grogra.graph.impl.Node newInstance ()
	{
		return new PhysicsModel ();
	}

	public boolean isDebugText ()
	{
		return debugText;
	}

	public void setDebugText (boolean value)
	{
		this.debugText = (boolean) value;
	}

	public ArrayList<MassObject> getMassObjects ()
	{
		return massObjects;
	}

	public void setMassObjects (ArrayList<MassObject> value)
	{
		massObjects$FIELD.setObject (this, value);
	}

//enh:end


}
